@@ -233,7 +233,7 @@ def is_ip(address):
         try:
             socket.inet_pton(family, address)
             return True
-        except (TypeError, OSError, IOError):
+        except (TypeError, ValueError, OSError, IOError):
             pass
     return False
 
@@ -271,15 +271,16 @@ class DNSResolver(object):
         self._cache = lru_cache.LRUCache(timeout=300)
         self._last_time = time.time()
         self._sock = None
+        self._servers = None
         self._parse_resolv()
         self._parse_hosts()
         # TODO monitor hosts change and reload hosts
         # TODO parse /etc/gai.conf and follow its rules
 
     def _parse_resolv(self):
+        self._servers = []
         try:
             with open('/etc/resolv.conf', 'rb') as f:
-                servers = []
                 content = f.readlines()
                 for line in content:
                     line = line.strip()
@@ -289,14 +290,11 @@ class DNSResolver(object):
                             if len(parts) >= 2:
                                 server = parts[1]
                                 if is_ip(server):
-                                    servers.append(server)
-                # TODO support more servers
-                if servers:
-                    self._dns_server = (servers[0], 53)
-                    return
+                                    self._servers.append(server)
         except IOError:
             pass
-        self._dns_server = ('8.8.8.8', 53)
+        if not self._servers:
+            self._servers = ['8.8.4.4', '8.8.8.8']
 
     def _parse_hosts(self):
         etc_path = '/etc/hosts'
@@ -328,13 +326,13 @@ class DNSResolver(object):
         loop.add(self._sock, eventloop.POLL_IN)
         loop.add_handler(self.handle_events)
 
-    def _call_callback(self, hostname, ip):
+    def _call_callback(self, hostname, ip, error=None):
         callbacks = self._hostname_to_cb.get(hostname, [])
         for callback in callbacks:
             if self._cb_to_hostname.__contains__(callback):
                 del self._cb_to_hostname[callback]
-            if ip:
-                callback((hostname, ip), None)
+            if ip or error:
+                callback((hostname, ip), error)
             else:
                 callback((hostname, None),
                          Exception('unknown hostname %s' % hostname))
@@ -377,7 +375,7 @@ class DNSResolver(object):
                 self._loop.add(self._sock, eventloop.POLL_IN)
             else:
                 data, addr = sock.recvfrom(1024)
-                if addr != self._dns_server:
+                if addr[0] not in self._servers:
                     logging.warn('received a packet other than our dns')
                     break
                 self._handle_data(data)
@@ -400,13 +398,14 @@ class DNSResolver(object):
                         del self._hostname_status[hostname]
 
     def _send_req(self, hostname, qtype):
-        logging.debug('resolving %s with type %d using server %s', hostname,
-                      qtype, self._dns_server)
         self._request_id += 1
         if self._request_id > 32768:
             self._request_id = 1
         req = build_request(hostname, qtype, self._request_id)
-        self._sock.sendto(req, self._dns_server)
+        for server in self._servers:
+            logging.debug('resolving %s with type %d using server %s', hostname,
+                          qtype, server)
+            self._sock.sendto(req, (server, 53))
 
     def resolve(self, hostname, callback):
         if not hostname:
